


function createComponentFunc(state) {
  return [
          state,
          -1
        ];
}

function addComponentFunc(state, param, param$1) {
  return state;
}

function removeComponentFunc(state, param, param$1) {
  return state;
}

function hasComponentFunc(state, param) {
  return false;
}

function getComponentFunc(state, param) {
  return -1;
}

function getGameObjectsFunc(state, param) {
  return [];
}

function setComponentDataFunc(state, param, param$1, param$2) {
  return state;
}

function getAllComponentsFunc(state) {
  return [];
}

function getNeedDisposedComponentsFunc(state) {
  return [];
}

function deferDisposeComponentFunc(state, param) {
  return state;
}

function disposeComponentsFunc(state, param) {
  return state;
}

function cloneComponentFunc(state, param, param$1, param$2) {
  return [
          state,
          []
        ];
}

export {
  createComponentFunc ,
  addComponentFunc ,
  removeComponentFunc ,
  hasComponentFunc ,
  getComponentFunc ,
  getGameObjectsFunc ,
  setComponentDataFunc ,
  getAllComponentsFunc ,
  getNeedDisposedComponentsFunc ,
  deferDisposeComponentFunc ,
  disposeComponentsFunc ,
  cloneComponentFunc ,
}
/* No side effect */
